@@ -76,6 +76,8 @@ def do_request(context, request_method, url, data):
     data = data and json.loads(data)
     try:
         r = request_executor.request(request_method, url, data)
+        if request_method == 'PATCH' and r.status == 409:
+            r = request_executor.request(request_method, url, data)
     except Exception:
         context.status_code = context.response = None
     else:
