@@ -2,7 +2,7 @@ import unittest
 
 from mock import Mock, patch
 from patroni.exceptions import PatroniException
-from patroni.utils import Retry, RetryFailedError, polling_loop, validate_directory
+from patroni.utils import Retry, RetryFailedError, enable_keepalive, find_executable, polling_loop, validate_directory
 
 
 class TestUtils(unittest.TestCase):
@@ -33,6 +33,23 @@ class TestUtils(unittest.TestCase):
     def test_validate_directory_is_not_a_directory(self):
         self.assertRaises(PatroniException, validate_directory, "/tmp")
 
+    def test_enable_keepalive(self):
+        with patch('socket.SIO_KEEPALIVE_VALS', 1, create=True):
+            self.assertIsNotNone(enable_keepalive(Mock(), 10, 5))
+        with patch('socket.SIO_KEEPALIVE_VALS', None, create=True):
+            for platform in ('linux2', 'darwin', 'other'):
+                with patch('sys.platform', platform):
+                    self.assertIsNone(enable_keepalive(Mock(), 10, 5))
+
+    @patch('sys.platform', 'win32')
+    def test_find_executable(self):
+        with patch('os.path.isfile', Mock(return_value=True)):
+            self.assertEqual(find_executable('vim'), 'vim.exe')
+        with patch('os.path.isfile', Mock(return_value=False)):
+            self.assertIsNone(find_executable('vim'))
+        with patch('os.path.isfile', Mock(side_effect=[False, True])):
+            self.assertEqual(find_executable('vim', '/'), '/vim.exe')
+
 
 @patch('time.sleep', Mock())
 class TestRetrySleeper(unittest.TestCase):
